@@ -2,11 +2,44 @@
 kombu.transport.SQS
 ===================
 
-Amazon SQS transport.
+Amazon SQS transport module for Kombu. This package implements an AMQP-like
+interface on top of Amazons SQS service, with the goal of being optimized for
+high performance and reliability.
 
+The default settings for this module are focused now on high performance in
+task queue situations where tasks are small, idempotent and run very fast.
+
+SQS Features supported by this transport:
+  Long Polling:
+    http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/
+      sqs-long-polling.html
+
+    Long polling is enabled by setting the `wait_time_seconds` transport
+    option to a number > 1. Amazon supports up to 20 seconds. This is
+    disabled for now, but will be enabled by default in the near future.
+
+  Batch API Actions:
+   http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/
+     sqs-batch-api.html
+
+    The default behavior of the SQS Channel.drain_events() method is to
+    request up to the 'prefetch_count' messages on every request to SQS.
+    These messages are stored locally in a deque object and passed back
+    to the Transport until the deque is empty, before triggering a new
+    API call to Amazon.
+
+    This behavior dramatically speeds up the rate that you can pull tasks
+    from SQS when you have short-running tasks (or a large number of workers).
+
+    When a Celery worker has multiple queues to monitor, it will pull down
+    up to 'prefetch_count' messages from queueA and work on them all before
+    moving on to queueB. If queueB is empty, it will wait up until
+    'polling_interval' expires before moving back and checking on queueA.
 """
+
 from __future__ import absolute_import
 
+import collections
 import socket
 import string
 
@@ -22,11 +55,15 @@ from boto.sqs.connection import SQSConnection
 from boto.sqs.message import Message
 
 from kombu.five import Empty, range, text_t
+from kombu.log import get_logger
 from kombu.utils import cached_property, uuid
 from kombu.utils.encoding import bytes_to_str, safe_str
+from kombu.transport.virtual import scheduling
 
 from . import virtual
 
+logger = get_logger(__name__)
+
 # dots are replaced by dash, all other punctuation
 # replaced by underscore.
 CHARS_REPLACE_TABLE = dict((ord(c), 0x5f)
@@ -149,11 +186,18 @@ class Channel(virtual.Channel):
             self._queue_cache[queue.name] = queue
         self._fanout_queues = set()
 
+        # The drain_events() method stores extra messages in a local
+        # Deque object. This allows multiple messages to be requested from
+        # SQS at once for performance, but maintains the same external API
+        # to the caller of the drain_events() method.
+        self._queue_message_cache = collections.deque()
+
     def basic_consume(self, queue, no_ack, *args, **kwargs):
         if no_ack:
             self._noack_queues.add(queue)
-        return super(Channel, self).basic_consume(queue, no_ack,
-                                                  *args, **kwargs)
+        return super(Channel, self).basic_consume(
+            queue, no_ack, *args, **kwargs
+        )
 
     def basic_cancel(self, consumer_tag):
         if consumer_tag in self._consumers:
@@ -161,12 +205,53 @@ class Channel(virtual.Channel):
             self._noack_queues.discard(queue)
         return super(Channel, self).basic_cancel(consumer_tag)
 
+    def drain_events(self, timeout=None):
+        """Return a single payload message from one of our queues.
+
+        :raises Empty: if no messages available.
+
+        """
+        # If we're not allowed to consume or have no consumers, raise Empty
+        if not self._consumers or not self.qos.can_consume():
+            raise Empty()
+        message_cache = self._queue_message_cache
+
+        # Check if there are any items in our buffer. If there are any, pop
+        # off that queue first.
+        try:
+            return message_cache.popleft()
+        except IndexError:
+            pass
+
+        # At this point, go and get more messages from SQS
+        res, queue = self._poll(self.cycle, timeout=timeout)
+        message_cache.extend((r, queue) for r in res)
+
+        # Now try to pop off the queue again.
+        try:
+            return message_cache.popleft()
+        except IndexError:
+            raise Empty()
+
+    def _reset_cycle(self):
+        """Reset the consume cycle.
+
+        :returns: a FairCycle object that points to our _get_bulk() method
+          rather than the standard _get() method. This allows for multiple
+          messages to be returned at once from SQS (based on the prefetch
+          limit).
+
+        """
+        self._cycle = scheduling.FairCycle(
+            self._get_bulk, self._active_queues, Empty,
+        )
+
     def entity_name(self, name, table=CHARS_REPLACE_TABLE):
         """Format AMQP queue name into a legal SQS queue name."""
         return text_t(safe_str(name)).translate(table)
 
     def _new_queue(self, queue, **kwargs):
-        """Ensures a queue exists in SQS."""
+        """Ensure a queue with given name exists in SQS."""
         # Translate to SQS name for consistency with initial
         # _queue_cache population.
         queue = self.entity_name(self.queue_name_prefix + queue)
@@ -212,10 +297,10 @@ class Channel(virtual.Channel):
 
     def _delete(self, queue, *args):
         """delete queue by name."""
-        self._queue_cache.pop(queue, None)
         if self.supports_fanout:
             self.table.queue_delete(queue)
         super(Channel, self)._delete(queue)
+        self._queue_cache.pop(queue, None)
 
     def exchange_delete(self, exchange, **kwargs):
         """Delete exchange by name."""
@@ -241,23 +326,86 @@ class Channel(virtual.Channel):
         for route in self.table.routes_for(exchange):
             self._put(route['queue'], message, **kwargs)
 
-    def _get(self, queue):
-        """Try to retrieve a single message off ``queue``."""
+    def _get_from_sqs(self, queue, count=1):
+        """Retrieve messages from SQS and returns the raw SQS message objects.
+
+        :returns: List of SQS message objects
+
+        """
         q = self._new_queue(queue)
         if W_LONG_POLLING and queue not in self._fanout_queues:
-            rs = q.get_messages(1, wait_time_seconds=self.wait_time_seconds)
+            return q.get_messages(
+                count, wait_time_seconds=self.wait_time_seconds,
+            )
         else:  # boto < 2.8
-            rs = q.get_messages(1)
-        if rs:
-            m = rs[0]
-            payload = loads(bytes_to_str(rs[0].get_body()))
-            if queue in self._noack_queues:
-                q.delete_message(m)
-            else:
-                payload['properties']['delivery_info'].update({
-                    'sqs_message': m, 'sqs_queue': q, })
-            return payload
-        raise Empty()
+            return q.get_messages(count)
+
+    def _message_to_python(self, message, queue_name, queue):
+        payload = loads(bytes_to_str(message.get_body()))
+        if queue_name in self._noack_queues:
+            queue.delete_message(message)
+        else:
+            payload['properties']['delivery_info'].update({
+                'sqs_message': message, 'sqs_queue': queue,
+            })
+        return payload
+
+    def _messages_to_python(self, messages, queue):
+        """Convert a list of SQS Message objects into Payloads.
+
+        This method handles converting SQS Message objects into
+        Payloads, and appropriately updating the queue depending on
+        the 'ack' settings for that queue.
+
+        :param messages: A list of SQS Message objects.
+        :param queue: String name representing the queue they came from
+
+        :returns: A list of Payload objects
+
+        """
+        q = self._new_queue(queue)
+        return [self._message_to_python(m, queue, q) for m in messages]
+
+    def _get_bulk(self, queue, max_if_unlimited=10):
+        """Try to retrieve multiple messages off ``queue``.
+
+        Where _get() returns a single Payload object, this method returns a
+        list of Payload objects. The number of objects returned is determined
+        by the total number of messages available in the queue and the
+        number of messages that the QoS object allows (based on the
+        prefetch_count).
+
+        .. note::
+            Ignores QoS limits so caller is responsible for checking
+            that we are allowed to consume at least one message from the
+            queue.  get_bulk will then ask QoS for an estimate of
+            the number of extra messages that we can consume.
+
+        args:
+            queue: The queue name (string) to pull from
+
+        returns:
+            payloads: A list of payload objects returned
+        """
+        # drain_events calls `can_consume` first, consuming
+        # a token, so we know that we are allowed to consume at least
+        # one message.
+        maxcount = self.qos.can_consume_max_estimate()
+        maxcount = max_if_unlimited if maxcount is None else max(maxcount, 1)
+        messages = self._get_from_sqs(queue, count=maxcount)
+
+        if not messages:
+            raise Empty()
+        return self._messages_to_python(messages, queue)
+
+    def _get(self, queue):
+        """Try to retrieve a single message off ``queue``."""
+        messages = self._get_from_sqs(queue, count=1)
+
+        if not messages:
+            raise Empty()
+
+        return self._messages_to_python(messages, queue)[0]
 
     def _restore(self, message,
                  unwanted_delivery_info=('sqs_message', 'sqs_queue')):
